@@ -1,12 +1,17 @@
 package oviewer
 
-import "github.com/gdamore/tcell/v2"
+import (
+	"github.com/gdamore/tcell/v2"
+)
 
 // setDelimiterMode sets the inputMode to Delimiter.
 func (root *Root) setDelimiterMode() {
 	input := root.input
 	input.value = ""
 	input.cursorX = 0
+
+	input.DelimiterCandidate.toLast(root.Doc.ColumnDelimiter)
+
 	input.Event = newDelimiterEvent(input.DelimiterCandidate)
 }
 
@@ -18,6 +23,7 @@ func delimiterCandidate() *candidate {
 			"\t",
 			"|",
 			",",
+			`/\s+/`,
 		},
 	}
 }
@@ -47,8 +53,7 @@ func (e *eventDelimiter) Prompt() string {
 // Confirm returns the event when the input is confirmed.
 func (e *eventDelimiter) Confirm(str string) tcell.Event {
 	e.value = str
-	e.clist.list = toLast(e.clist.list, str)
-	e.clist.p = 0
+	e.clist.toLast(str)
 	e.SetEventNow()
 	return e
 }
