@@ -1,12 +1,19 @@
 package oviewer
 
-import "github.com/gdamore/tcell/v2"
+import (
+	"github.com/gdamore/tcell/v2"
+)
 
 // setSearchMode sets the inputMode to Search.
 func (root *Root) setSearchMode() {
 	input := root.input
 	input.value = ""
 	input.cursorX = 0
+
+	if root.searcher != nil {
+		input.SearchCandidate.toLast(root.searcher.String())
+	}
+
 	input.Event = newSearchEvent(input.SearchCandidate)
 	root.OriginPos = root.Doc.topLN
 }
@@ -47,21 +54,20 @@ func (e *eventInputSearch) Prompt() string {
 // Confirm returns the event when the input is confirmed.
 func (e *eventInputSearch) Confirm(str string) tcell.Event {
 	e.value = str
-	e.clist.list = toLast(e.clist.list, str)
-	e.clist.p = 0
+	e.clist.toLast(str)
 	e.SetEventNow()
 	return e
 }
 
 // Up returns strings when the up key is pressed during input.
 func (e *eventInputSearch) Up(str string) string {
-	e.clist.list = toAddLast(e.clist.list, str)
+	e.clist.toAddLast(str)
 	return e.clist.up()
 }
 
 // Down returns strings when the down key is pressed during input.
 func (e *eventInputSearch) Down(str string) string {
-	e.clist.list = toAddTop(e.clist.list, str)
+	e.clist.toAddTop(str)
 	return e.clist.down()
 }
 
@@ -70,6 +76,11 @@ func (root *Root) setBackSearchMode() {
 	input := root.input
 	input.value = ""
 	input.cursorX = 0
+
+	if root.searcher != nil {
+		input.SearchCandidate.toLast(root.searcher.String())
+	}
+
 	input.Event = newBackSearchEvent(input.SearchCandidate)
 	root.OriginPos = root.Doc.topLN
 }
@@ -98,26 +109,28 @@ func (e *eventInputBackSearch) Prompt() string {
 // Confirm returns the event when the input is confirmed.
 func (e *eventInputBackSearch) Confirm(str string) tcell.Event {
 	e.value = str
-	e.clist.list = toLast(e.clist.list, str)
-	e.clist.p = 0
+	e.clist.toLast(str)
 	e.SetEventNow()
 	return e
 }
 
 // Up returns strings when the up key is pressed during input.
 func (e *eventInputBackSearch) Up(str string) string {
-	e.clist.list = toAddLast(e.clist.list, str)
+	e.clist.toAddLast(str)
 	return e.clist.up()
 }
 
 // Down returns strings when the down key is pressed during input.
 func (e *eventInputBackSearch) Down(str string) string {
-	e.clist.list = toAddTop(e.clist.list, str)
+	e.clist.toAddTop(str)
 	return e.clist.down()
 }
 
 // searchCandidates returns the list of search candidates.
 func (root *Root) searchCandidates(n int) []string {
+	root.input.SearchCandidate.mux.Lock()
+	defer root.input.SearchCandidate.mux.Unlock()
+
 	listLen := len(root.input.SearchCandidate.list)
 	start := max(0, listLen-n)
 	return root.input.SearchCandidate.list[start:listLen]
