@@ -161,6 +161,8 @@ type OVPromptConfigNormal struct {
 	ShowFilename bool
 	// InvertColor controls whether the text is colored and inverted.
 	InvertColor bool
+	// ProcessOfCount controls whether to display the progress of the count.
+	ProcessOfCount bool
 }
 
 // OVPromptConfig is the prompt setting.
@@ -914,7 +916,7 @@ func (root *Root) WriteOriginal() {
 		m.bottomLN = m.BufEndNum()
 	}
 
-	start := max(0, m.topLN-root.BeforeWriteOriginal)
+	start := max(0, m.topLN-(m.sectionHeaderNum+root.BeforeWriteOriginal))
 	end := m.bottomLN - 1
 	if root.AfterWriteOriginal != 0 {
 		end = m.topLN + root.AfterWriteOriginal - 1
