@@ -12,6 +12,7 @@ import (
 	"strconv"
 	"strings"
 	"sync/atomic"
+	"unicode"
 
 	"code.rocketnine.space/tslocum/cbind"
 	"github.com/gdamore/tcell/v2"
@@ -34,8 +35,14 @@ import (
 // Searcher interface provides a match method that determines
 // if the search word matches the argument string.
 type Searcher interface {
+	// Match searches for bytes.
 	Match([]byte) bool
+	// MatchString searches for strings.
 	MatchString(string) bool
+	// FindAll searches for strings and returns the index of the match.
+	FindAll(string) [][]int
+	// String returns the search word.
+	String() string
 }
 
 // searchWord is a case-insensitive search.
@@ -43,50 +50,82 @@ type searchWord struct {
 	word string
 }
 
-// searchWord Match is a case-insensitive search.
+// searchWord Match is a case-insensitive search for bytes.
 func (substr searchWord) Match(s []byte) bool {
 	s = stripEscapeSequenceBytes(s)
 	return bytes.Contains(bytes.ToLower(s), []byte(substr.word))
 }
 
-// searchWord MatchString is a case-insensitive search.
+// searchWord MatchString is a case-insensitive search for string.
 func (substr searchWord) MatchString(s string) bool {
 	s = stripEscapeSequenceString(s)
 	return strings.Contains(strings.ToLower(s), substr.word)
 }
 
+// searchWord FindAll searches for strings and returns the index of the match.
+func (substr searchWord) FindAll(s string) [][]int {
+	s = strings.ToLower(s)
+	return allStringIndex(s, substr.word)
+}
+
+// searchWord String returns the search word.
+func (substr searchWord) String() string {
+	return substr.word
+}
+
 // sensitiveWord is a case-sensitive search.
 type sensitiveWord struct {
 	word string
 }
 
-// sensitiveWord Match is a case-sensitive search.
+// sensitiveWord Match is a case-sensitive search for bytes.
 func (substr sensitiveWord) Match(s []byte) bool {
 	s = stripEscapeSequenceBytes(s)
 	return bytes.Contains(s, []byte(substr.word))
 }
 
-// sensitiveWord Match is a case-sensitive search.
+// sensitiveWord MatchString is a case-sensitive search for string.
 func (substr sensitiveWord) MatchString(s string) bool {
 	s = stripEscapeSequenceString(s)
 	return strings.Contains(s, substr.word)
 }
 
+// sensitiveWord FindAll searches for strings and returns the index of the match.
+func (substr sensitiveWord) FindAll(s string) [][]int {
+	return allStringIndex(s, substr.word)
+}
+
+// String returns the search word.
+func (substr sensitiveWord) String() string {
+	return substr.word
+}
+
 // regexpWord is a regular expression search.
 type regexpWord struct {
-	word *regexp.Regexp
+	word   string
+	regexp *regexp.Regexp
 }
 
-// regexpWord Match is a regular expression search.
+// regexpWord Match is a regular expression search for bytes.
 func (substr regexpWord) Match(s []byte) bool {
 	s = stripEscapeSequenceBytes(s)
-	return substr.word.Match(s)
+	return substr.regexp.Match(s)
 }
 
-// regexpWord Match is a regular expression search.
+// regexpWord MatchString is a regular expression search for string.
 func (substr regexpWord) MatchString(s string) bool {
 	s = stripEscapeSequenceString(s)
-	return substr.word.MatchString(s)
+	return substr.regexp.MatchString(s)
+}
+
+// regexpWord FindAll searches for strings and returns the index of the match.
+func (substr regexpWord) FindAll(s string) [][]int {
+	return substr.regexp.FindAllStringIndex(s, -1)
+}
+
+// regexpWord String returns the search word.
+func (substr regexpWord) String() string {
+	return substr.word
 }
 
 // stripRegexpES is a regular expression that excludes escape sequences.
@@ -114,7 +153,8 @@ func stripEscapeSequenceBytes(s []byte) []byte {
 func NewSearcher(word string, searchReg *regexp.Regexp, caseSensitive bool, regexpSearch bool) Searcher {
 	if regexpSearch && word != regexp.QuoteMeta(word) {
 		return regexpWord{
-			word: searchReg,
+			word:   word,
+			regexp: searchReg,
 		}
 	}
 	if caseSensitive {
@@ -135,9 +175,9 @@ func NewSearcher(word string, searchReg *regexp.Regexp, caseSensitive bool, rege
 
 // regexpCompile is regexp.Compile the search string.
 func regexpCompile(r string, caseSensitive bool) *regexp.Regexp {
-	opt := "(?m)"
+	opt := ""
 	if !caseSensitive {
-		opt = "(?mi)"
+		opt = "(?i)"
 	}
 	re, err := regexp.Compile(opt + r)
 	if err == nil {
@@ -186,13 +226,17 @@ func condRegexpCompile(in string) *regexp.Regexp {
 // searchPosition returns the position where the search in the argument line matched.
 // searchPosition uses cache.
 func (root *Root) searchPosition(lN int, str string) [][]int {
-	var indexes [][]int
-	if root.Config.RegexpSearch {
-		indexes = searchPositionReg(str, root.searchReg)
-	} else {
-		indexes = searchPositionStr(root.Config.CaseSensitive, str, root.searchWord)
+	return root.searcher.FindAll(str)
+}
+
+// searchXPos returns the x position of the first match.
+func (root *Root) searchXPos(lN int) int {
+	line, _ := root.Doc.getLineC(lN, root.Doc.TabWidth)
+	indexes := root.searchPosition(lN, line.str)
+	if len(indexes) == 0 {
+		return 0
 	}
-	return indexes
+	return line.pos.x(indexes[0][0])
 }
 
 // searchPositionReg returns an array of the beginning and end of the string
@@ -222,23 +266,35 @@ func searchPositionStr(caseSensitive bool, s string, substr string) [][]int {
 // Returns nil if there is no search term.
 func (root *Root) setSearcher(word string, caseSensitive bool) Searcher {
 	if word == "" {
-		root.searchWord = ""
-		root.searchReg = nil
+		root.searcher = nil
 		return nil
 	}
 	root.input.value = word
-	root.searchWord = word
-	root.searchReg = regexpCompile(root.searchWord, caseSensitive)
 
-	return NewSearcher(root.searchWord, root.searchReg, caseSensitive, root.Config.RegexpSearch)
+	if root.Config.SmartCaseSensitive {
+		for _, ch := range word {
+			if unicode.IsUpper(ch) {
+				caseSensitive = true
+				break
+			}
+		}
+	}
+	reg := regexpCompile(word, caseSensitive)
+	searcher := NewSearcher(word, reg, caseSensitive, root.Config.RegexpSearch)
+	root.searcher = searcher
+	return searcher
 }
 
 // searchMove searches forward/backward and moves to the nearest matching line.
 func (root *Root) searchMove(ctx context.Context, forward bool, lN int, searcher Searcher) {
 	if searcher == nil {
+		if root.Doc.jumpTargetSection {
+			root.Doc.jumpTargetNum = 0
+		}
 		return
 	}
-	root.setMessagef("search:%v (%v)Cancel", root.searchWord, strings.Join(root.cancelKeys, ","))
+	word := root.searcher.String()
+	root.setMessagef("search:%v (%v)Cancel", word, strings.Join(root.cancelKeys, ","))
 	eg, ctx := errgroup.WithContext(ctx)
 	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
@@ -248,38 +304,36 @@ func (root *Root) searchMove(ctx context.Context, forward bool, lN int, searcher
 	})
 
 	eg.Go(func() error {
-		var err error
-		if forward {
-			lN, err = root.Doc.SearchLine(ctx, searcher, lN)
-		} else {
-			lN, err = root.Doc.BackSearchLine(ctx, searcher, lN)
-		}
-		root.searchQuit()
+		n, err := root.Doc.searchLine(ctx, searcher, forward, lN)
+		root.sendSearchQuit()
 		if err != nil {
-			startNum := root.Doc.BufStartNum()
-			if lN < startNum {
-				// If lN is before startNum, move to startNum.
-				root.searchGo(startNum)
-				log.Printf("Moved to %d because %d is out of range.", startNum, lN)
-				return fmt.Errorf("search moved to %d:%w:%v", startNum, err, root.searchWord)
-			}
-			return fmt.Errorf("search:%w:%v", err, root.searchWord)
+			return fmt.Errorf("search:%w:%v", err, word)
 		}
-		root.searchGo(lN)
+		root.sendSearchMove(n)
 		return nil
 	})
 
 	if err := eg.Wait(); err != nil {
-		root.setMessage(err.Error())
+		root.setMessageLog(err.Error())
 		return
 	}
-	root.setMessagef("search:%v", root.searchWord)
+	root.setMessagef("search:%v", word)
+}
+
+// searchLine is a forward/backward search wrap function
+func (m *Document) searchLine(ctx context.Context, searcher Searcher, forward bool, lN int) (int, error) {
+	if forward {
+		return m.SearchLine(ctx, searcher, lN)
+	}
+	return m.BackSearchLine(ctx, searcher, lN)
 }
 
 // Search searches for the search term and moves to the nearest matching line.
 func (m *Document) Search(ctx context.Context, searcher Searcher, chunkNum int, line int) (int, error) {
 	if !m.seekable {
-		m.requestLoad(chunkNum)
+		if chunkNum != 0 && m.store.lastChunkNum() <= chunkNum {
+			m.requestLoad(chunkNum)
+		}
 	} else {
 		if m.store.lastChunkNum() < chunkNum {
 			return 0, ErrOutOfChunk
@@ -355,9 +409,8 @@ func (m *Document) SearchLine(ctx context.Context, searcher Searcher, lN int) (i
 		default:
 		}
 
-		// endChunk may be updated by Search.
-		endChunk := (m.BufEndNum() - 1) / ChunkSize
-		if cn >= endChunk {
+		// lastChunkNum may be updated by Search.
+		if cn >= m.store.lastChunkNum() {
 			break
 		}
 		sn = 0
@@ -418,7 +471,7 @@ func (root *Root) cancelWait(cancel context.CancelFunc) error {
 		case *eventUpdateEndNum:
 			root.updateEndNum()
 		default:
-			log.Printf("unexpected event %#v", ev)
+			root.setMessageLogf("unexpected event %#v", ev)
 			return nil
 		}
 	}
@@ -429,8 +482,8 @@ type eventSearchQuit struct {
 	tcell.EventTime
 }
 
-// searchQuit fires the eventSearchQuit event.
-func (root *Root) searchQuit() {
+// sendSearchQuit fires the eventSearchQuit event.
+func (root *Root) sendSearchQuit() {
 	ev := &eventSearchQuit{}
 	ev.SetEventNow()
 	root.postEvent(ev)
@@ -442,7 +495,7 @@ type eventSearchMove struct {
 	value int
 }
 
-func (root *Root) searchGo(lN int) {
+func (root *Root) sendSearchMove(lN int) {
 	ev := &eventSearchMove{}
 	ev.SetEventNow()
 	ev.value = lN
@@ -474,17 +527,12 @@ func (root *Root) incSearch(ctx context.Context, forward bool, lN int) {
 
 	ctx = root.cancelRestart(ctx)
 	go func() {
-		var err error
-		if forward {
-			lN, err = root.Doc.SearchLine(ctx, searcher, lN)
-		} else {
-			lN, err = root.Doc.BackSearchLine(ctx, searcher, lN)
-		}
+		n, err := root.Doc.searchLine(ctx, searcher, forward, lN)
 		if err != nil {
 			root.debugMessage(fmt.Sprintf("incSearch: %s", err))
 			return
 		}
-		root.searchGo(lN)
+		root.sendSearchMove(n)
 	}()
 }
 
@@ -500,17 +548,21 @@ func (root *Root) cancelRestart(ctx context.Context) context.Context {
 
 // returnStartPosition checks the input value and returns the start position.
 func (root *Root) returnStartPosition() int {
+	word := ""
+	if root.searcher != nil {
+		word = root.searcher.String()
+	}
 	start := root.Doc.topLN
-	if !strings.Contains(root.searchWord, root.OriginStr) {
+	if !strings.Contains(word, root.OriginStr) {
 		start = root.OriginPos
 	}
-	root.OriginStr = root.searchWord
+	root.OriginStr = word
 	return start
 }
 
 // startSearchLN returns the start position of the search.
 func (root *Root) startSearchLN() int {
-	l := root.scr.lineNumber(root.headerLen + root.Doc.JumpTarget)
+	l := root.scr.lineNumber(root.Doc.headerLen + root.Doc.jumpTargetNum)
 	if l.number-root.Doc.topLN > root.Doc.topLN {
 		return 0
 	}
@@ -526,21 +578,21 @@ func (root *Root) firstSearch(ctx context.Context) {
 // nextSearch performs the next search.
 func (root *Root) nextSearch(ctx context.Context, str string) {
 	searcher := root.setSearcher(str, root.Config.CaseSensitive)
-	l := root.scr.lineNumber(root.headerLen + root.Doc.JumpTarget)
+	l := root.scr.lineNumber(root.Doc.headerLen + root.Doc.jumpTargetNum)
 	root.searchMove(ctx, true, l.number+1, searcher)
 }
 
 // firstBackSearch performs the first back search immediately after the input.
 func (root *Root) firstBackSearch(ctx context.Context) {
 	searcher := root.setSearcher(root.input.value, root.Config.CaseSensitive)
-	l := root.scr.lineNumber(root.headerLen)
+	l := root.scr.lineNumber(root.Doc.headerLen)
 	root.searchMove(ctx, false, l.number, searcher)
 }
 
 // nextBackSearch performs the next back search.
 func (root *Root) nextBackSearch(ctx context.Context, str string) {
 	searcher := root.setSearcher(str, root.Config.CaseSensitive)
-	l := root.scr.lineNumber(root.headerLen + root.Doc.JumpTarget)
+	l := root.scr.lineNumber(root.Doc.headerLen + root.Doc.jumpTargetNum)
 	root.searchMove(ctx, false, l.number-1, searcher)
 }
 
@@ -550,10 +602,14 @@ type eventNextSearch struct {
 	str string
 }
 
-// setNextSearch fires the eventNextSearch event.
-func (root *Root) setNextSearch() {
+// sendNextSearch fires the eventNextSearch event.
+func (root *Root) sendNextSearch() {
+	if root.searcher == nil {
+		return
+	}
+
 	ev := &eventNextSearch{}
-	ev.str = root.searchWord
+	ev.str = root.searcher.String()
 	ev.SetEventNow()
 	root.postEvent(ev)
 }
@@ -564,10 +620,14 @@ type eventNextBackSearch struct {
 	str string
 }
 
-// setNextBackSearch fires the eventNextBackSearch event.
-func (root *Root) setNextBackSearch() {
+// sendNextBackSearch fires the eventNextBackSearch event.
+func (root *Root) sendNextBackSearch() {
+	if root.searcher == nil {
+		return
+	}
+
 	ev := &eventNextBackSearch{}
-	ev.str = root.searchWord
+	ev.str = root.searcher.String()
 	ev.SetEventNow()
 	root.postEvent(ev)
 }
@@ -576,6 +636,10 @@ func (root *Root) setNextBackSearch() {
 // This is for calling Search from the outside.
 // Normally, the event is executed from Confirm.
 func (root *Root) Search(str string) {
+	root.sendForwardSearch(str)
+}
+
+func (root *Root) sendForwardSearch(str string) {
 	if !root.checkScreen() {
 		return
 	}
@@ -589,6 +653,10 @@ func (root *Root) Search(str string) {
 // This is for calling Search from the outside.
 // Normally, the event is executed from Confirm.
 func (root *Root) BackSearch(str string) {
+	root.sendBackSearch(str)
+}
+
+func (root *Root) sendBackSearch(str string) {
 	if !root.checkScreen() {
 		return
 	}
@@ -603,7 +671,7 @@ func (m *Document) searchChunk(chunkNum int, searcher Searcher) (int, error) {
 	// Seek to the start of the chunk.
 	chunk := m.store.chunks[chunkNum]
 	if _, err := m.file.Seek(chunk.start, io.SeekStart); err != nil {
-		return 0, fmt.Errorf("seek:%w", err)
+		return 0, fmt.Errorf("seek: %w", err)
 	}
 
 	// Read the chunk line by line.
@@ -622,7 +690,7 @@ func (m *Document) searchChunk(chunkNum int, searcher Searcher) (int, error) {
 
 		// If the line is complete, check if it matches.
 		if !isPrefix {
-			if searcher.Match(line.Bytes()) {
+			if searcher.Match(bytes.TrimSuffix(line.Bytes(), []byte("\n"))) {
 				return num, nil
 			}
 			num++
